// Copyright 2006, by the California Institute of 
// Technology. ALL RIGHTS RESERVED. United States Government 
// Sponsorship acknowledged. Any commercial use must be negotiated with 
// the Office of Technology Transfer at the California Institute of 
// Technology.
//
// This software may be subject to U.S. export control laws. By 
// accepting this software, the user agrees to comply with all 
// applicable U.S. export laws and regulations. User has the 
// responsibility to obtain export licenses, or other export authority 
// as may be required before exporting such information to foreign 
// countries or providing access to foreign persons.
//
// $Id: ElementDefinitionTest.java 2937 2007-09-27 20:20:25Z pramirez $ 
//

package gov.nasa.pds.tools.dict;

import gov.nasa.pds.tools.dict.ElementDefinition;

import java.util.ArrayList;
import java.util.List;

import junit.framework.*;

/**
 * @author pramirez
 * @version $Revision: 2937 $
 * 
 */
public class ElementDefinitionTest extends TestCase {
    public ElementDefinitionTest(String name) {
        super(name);
    }
    
    public void testCtor() {
        ElementDefinition definition = new ElementDefinition("TEST");
        
        assertEquals("TEST", definition.getIdentifier());
        assertEquals("NULL", definition.getDataType());
        assertEquals("NONE", definition.getUnitId());
        assertEquals("NONE", definition.getValueType());
        assertEquals(0, definition.getMinLength());
        assertEquals(Integer.MAX_VALUE, definition.getMaxLength());
        assertEquals(0, definition.getValues().size());
    }
    
    public void testSetters() {
        ElementDefinition definition = new ElementDefinition("TEST");
        
        assertEquals("TEST", definition.getIdentifier());
        definition.setIdentifier("TEST_NEW");
        assertEquals("TEST_NEW", definition.getIdentifier());
        
        assertEquals("NULL", definition.getDataType());
        definition.setDataType("DATA_TYPE");
        assertEquals("DATA_TYPE", definition.getDataType());
        
        assertEquals("NONE", definition.getUnitId());
        definition.setUnitId("m**2");
        assertEquals("m**2", definition.getUnitId());
        
        assertEquals("NONE", definition.getValueType());
        definition.setValueType("INTEGER");
        assertEquals("INTEGER", definition.getValueType());
        
        assertEquals(0, definition.getMinLength());
        definition.setMinLength(10);
        assertEquals(10, definition.getMinLength());
        
        assertEquals(Integer.MAX_VALUE, definition.getMaxLength());
        definition.setMaxLength(100);
        assertEquals(100, definition.getMaxLength());
        
        assertEquals(0, definition.getValues().size());
        List values = new ArrayList();
        values.add("TEST");
        definition.setValues(values);
        assertEquals("TEST", ((ArrayList) definition.getValues()).get(0));
    }
}
